import numpy as np
import pyopencl as cl

from meshmode.mesh.io import read_gmsh
from meshmode.discretization import Discretization
from meshmode.discretization.poly_element import \
        InterpolatoryQuadratureSimplexGroupFactory

cl_ctx = cl.create_some_context()
queue = cl.CommandQueue(cl_ctx)

# Get target
from pytential.target import PointsTarget
target_mesh = read_gmsh('enclosing_cube.msh')
target = PointsTarget(target_mesh.vertices)

# Read mesh and make source discretization
source_mesh = read_gmsh('enclosed_sphere.msh')
ambient_dim = 3
bdry_quad_order = 1
fine_order = 1
qbx_order = 1
fmm_order = 3

pre_density_discr = Discretization(
    cl_ctx,
    source_mesh,
    InterpolatoryQuadratureSimplexGroupFactory(bdry_quad_order))

from pytential.qbx import QBXLayerPotentialSource

# Make qbx -- nb that this bug only appears when fmm_backend='fmmlib' is set
qbx, _ = QBXLayerPotentialSource(
        pre_density_discr, fine_order=fine_order, qbx_order=qbx_order,
        fmm_order=fmm_order, fmm_backend='fmmlib',
        ).with_refinement()
density_discr = qbx.density_discr

from pytential import sym, bind
from sumpy.kernel import LaplaceKernel

# Bind layer potential operator
op = sym.S(LaplaceKernel(ambient_dim),
           sym.var("sigma"),
           qbx_forced_limit=None)
bound_op = bind((qbx, target), op)

sigma = cl.array.to_device(queue, np.zeros(density_discr.nnodes))
bound_op(queue, sigma=sigma)
